#!/usr/bin/env python

# History:
#
# 6/21/02 -- Added ability to generate controller for an
#   Albatross object-based application.

import sys
import getopt
from xml.dom import minidom
from xml.dom import Node

#
# If you have installed IPython you can uncomment and use the following.
# IPython is available from http://www-hep.colorado.edu/~fperez/ipython.
#
#from IPython.Shell import IPythonShellEmbed
#IPShell = IPythonShellEmbed('-nosep',
#    banner = 'Entering interpreter.  Ctrl-D to exit.',
#    exit_msg = 'Leaving Interpreter.')

# Use the following line where and when you want to drop into the
# IPython shell:
#    IPShell(vars(), '<a msg>')

#
# Support/utility functions.
#

def showIndent(outfile, level):
    for idx in range(level):
        outfile.write('    ')


#
# Data representation classes.
#

class fsm:
    def __init__(self, name='', lastmodified='', description='', \
            currentstate='', startstatename='', \
            endstatenames=None, states=None, \
            startstate=None, endstates=None):
        self.name = name
        self.lastmodified = lastmodified
        self.description = description
        self.currentstate = currentstate
        self.startstatename = startstatename
        self.startstate = startstate
        self.endstatenames = endstatenames
        self.endstates = []
##         if endstates is None:
##             self.endstates = []
##         else:
##             self.endstates = endstates
        if states is None:
            self.states = []
        else:
            self.states = states
        self.stateDict = {}
        #self.stateNameDict = {}
    def getName(self): return self.name
    def setName(self, name): self.name = name
    def getLastmodified(self): return self.lastmodified
    def setLastmodified(self, lastmodified): self.lastmodified = lastmodified
    def getDescription(self): return self.description
    def setDescription(self, description): self.description = description
    def getCurrentstate(self): return self.currentstate
    def setCurrentstate(self, currentstate): self.currentstate = currentstate
    def getStartstatename(self): return self.startstatename
    def setStartstatename(self, startstatename):
        self.startstatename = startstatename
    def getStartstate(self): return self.startstate
    def setStartstate(self, startstate): self.startstate = startstate
    def getEndstatenames(self): return self.endstatenames
    def setEndstatenames(self, endstatenames):
        self.endstatenames = endstatenames
    def getStates(self): return self.states
    def addStates(self, states): self.states.append(states)
    def getStateFromName(self, name):
        if self.stateDict.has_key(name):
            return self.stateDict[name]
        else:
            return None
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<fsm>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<name>%s</name>\n' % self.getName())
        showIndent(outfile, level)
        outfile.write('<lastmodified>%s</lastmodified>\n' % \
            self.getLastmodified())
        showIndent(outfile, level)
        outfile.write('<description>%s</description>\n' % \
            self.getDescription())
        showIndent(outfile, level)
        outfile.write('<currentstate>%s</currentstate>\n' % \
            self.getCurrentstate())
        showIndent(outfile, level)
        outfile.write('<startstatename>%s</startstatename>\n' % \
            self.getStartstatename())
        self.endstatenames.export(outfile, level)
        for states in self.states:
            states.export(outfile, level)
        level -= 1
        showIndent(outfile, level)
        outfile.write('</fsm>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'name':
                name = ''
                for text_ in child.childNodes:
                    name += text_.nodeValue
                self.name = name
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'lastmodified':
                lastmodified = ''
                for text_ in child.childNodes:
                    lastmodified += text_.nodeValue
                self.lastmodified = lastmodified
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'description':
                description = ''
                for text_ in child.childNodes:
                    description += text_.nodeValue
                self.description = description
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'currentstate':
                currentstate = ''
                for text_ in child.childNodes:
                    currentstate += text_.nodeValue
                self.currentstate = currentstate
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'startstatename':
                startstatename = ''
                for text_ in child.childNodes:
                    startstatename += text_.nodeValue
                self.startstatename = startstatename
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'endstatenames':
                obj = endstatenames()
                obj.build(child)
                self.setEndstatenames(obj)
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'state':
                obj = state()
                obj.build(child)
                self.states.append(obj)
    def step(self):
        pass
    def run(self):
        while not self.currentstate in self.endstates:
            self.currentstate = self.currentstate.step(self.environment)
    def linkStates(self):
        # Create a dictionary mapping state names to states.
        # Create another dictionary mapping states to state names.
        self.stateDict = stateDict = {}
        #self.stateNameDict = stateNameDict = {}
        for state in self.states:
            stateDict[state.getName()] = state
            #stateNameDict[state] = state.getName()
        # Link the transitions.
        for state in self.states:
            state.linkTransitions(stateDict)
        # Link the startstate.
        if stateDict.has_key(self.startstatename):
            self.startstate = stateDict[self.startstatename]
        # Link the endstates.
        self.endstates = []
        for name in self.endstatenames.getEndstatenames():
            if stateDict.has_key(name):
                self.endstates.append(stateDict[name])
    def show(self):
        print 'FSM: %s' % self.name
        for state in self.endstates:
            print '   End state: %s -- %s' % (state, state.getName())
        for state in self.states:
            state.show()
    def getNextState(self, currentState, argsDict):
        nextState = currentState.getNextState(argsDict)
        return nextState
    def reportTransitions(self, outfile, level):
        outfile.write('FSM: %s\n' % self.name)
        level += 1
        for states in self.states:
            states.reportTransitions(outfile, level, self.stateDict)
    def reportPageTransitions(self, outfile, level):
        outfile.write('FSM: %s\n' % self.name)
        level += 1
        for states in self.states:
            states.reportPageTransitions(outfile, level, self.stateDict)
    def reportStates(self, outfile, level):
        outfile.write('FSM: %s\n' % self.name)
        level += 1
        for states in self.states:
            states.reportStates(outfile, level, self.stateDict)
    def generateObjCtrlr(self, outfile, level, pages=1):
        #level += 1
        for states in self.states:
            states.generateObjCtrlr(outfile, level, \
                self.stateDict, self.endstates)
            if pages:
                for state in self.getStates():
                    state.generateHtml(self.stateDict)
    def generateModCtrlr(self, outfile, level, pages=1):
        for state in self.getStates():
            outFileName = '%s.py' % state.getPagename()
            outfile = file(outFileName, 'w')
            state.generateModCtrlr(outfile, level, \
                self.stateDict, self.endstates)
            outfile.close()
            if pages:
                for state in self.getStates():
                    state.generateHtml(self.stateDict)
# end class fsm


class state:
    def __init__(self, name='', pagename= '', transitions=None):
        self.name = name
        self.pagename = pagename
        if transitions is None:
            self.transitions = []
        else:
            self.transitions = transitions
    def getName(self): return self.name
    def setName(self, name): self.name = name
    def getPagename(self): return self.pagename
    def setPagename(self, pagename): self.pagename = pagename
    def getTransitions(self): return self.transitions
    def addTransitions(self, transitions): self.transitions.append(transitions)
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<state>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<name>%s</name>\n' % self.getName())
        showIndent(outfile, level)
        outfile.write('<pagename>%s</pagename>\n' % self.getPagename())
        for transitions in self.transitions:
            transitions.export(outfile, level)
        level -= 1
        showIndent(outfile, level)
        outfile.write('</state>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'name':
                name = ''
                for text_ in child.childNodes:
                    name += text_.nodeValue
                self.name = name
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'pagename':
                pagename = ''
                for text_ in child.childNodes:
                    pagename += text_.nodeValue
                self.pagename = pagename
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'transition':
                obj = transition()
                obj.build(child)
                self.transitions.append(obj)
    def step(self, inputs, environment):
        newState = None
        for transition in self.transitions:
            if transition.test(inputs):
                newState = transition.transition()
                return newState
    def linkTransitions(self, stateDict):
        for transition in self.transitions:
            transition.linkTransitions(stateDict)
    def show(self):
        print 'State: %s' % self.name
        for transition in self.transitions:
            print '    Transition -- newstate: %s -- %s' % \
                  (transition.getNewstate(), \
                   transition.getNewstate().getName())
    def getNextState(self, argsDict):
        for transition in self.transitions:
            if transition.checkInputs(argsDict):
                return transition.getNewstate()
        return None
    def reportTransitions(self, outfile, level, stateDict):
        showIndent(outfile, level)
        outfile.write('State: %s  Page: %s\n' % (self.name, self.pagename))
        level += 1
        for transitions in self.transitions:
            transitions.reportTransitions(outfile, level, stateDict)
    def reportPageTransitions(self, outfile, level, stateDict):
        showIndent(outfile, level)
        outfile.write('Page: %s\n' % self.pagename)
        level += 1
        for transitions in self.transitions:
            transitions.reportPageTransitions(outfile, level, stateDict)
    def reportStates(self, outfile, level, stateDict):
        showIndent(outfile, level)
        outfile.write('State: %s  Page: %s\n' % (self.name, self.pagename))
    def generateObjCtrlr(self, outfile, level, stateDict, endstates):
        showIndent(outfile, level)
        outfile.write('class %s:\n' % (self.pagename,))
        level += 1
        showIndent(outfile, level)
        outfile.write("name = '%s'\n" % self.pagename)
        if len(self.transitions) > 0:
            showIndent(outfile, level)
            outfile.write('def page_process(self, ctx):\n')
            level += 1
            firstTime = 1
            for transitions in self.transitions:
                transitions.generateObjCtrlr(outfile, level, \
                    stateDict, firstTime)
                firstTime = 0
            level -= 1
        outfile.write('\n')
        showIndent(outfile, level)
        outfile.write('def page_display(self, ctx):\n')
        level += 1
        showIndent(outfile, level)
        outfile.write("ctx.run_template('%s.html')\n" % self.pagename)
        if self in endstates:
            showIndent(outfile, level)
            outfile.write('ctx.set_save_session(0)\n')
            showIndent(outfile, level)
            outfile.write('ctx.remove_session()\n')
        outfile.write("\n\n")
    def generateModCtrlr(self, outfile, level, stateDict, endstates):
        wrt = outfile.write
        wrt("import time\n")
        wrt("\n")
        wrt("DEBUG_FLAG = 0\n")
        wrt("\n")
        wrt("def pdebug(appid, msg):\n")
        wrt("    if DEBUG_FLAG:\n")
        wrt("        of = file('/w2/Tmp/Logs/test.log', 'a')\n")
        wrt("        of.write('[%s][%s] %s' % (appid, time.ctime(), msg))\n")
        wrt("        of.close()\n")
        wrt("\n")
        wrt("\n")
        wrt("name = '%s'\n" % self.getPagename())
        wrt("\n")
        wrt("\n")
        if len(self.transitions) > 0:
            showIndent(outfile, level)
            outfile.write('def page_process(ctx):\n')
            level += 1
            firstTime = 1
            for transitions in self.transitions:
                transitions.generateModCtrlr(outfile, level, \
                    stateDict, firstTime)
                firstTime = 0
            level -= 1
        outfile.write('\n')
        showIndent(outfile, level)
        outfile.write('def page_display(ctx):\n')
        level += 1
        showIndent(outfile, level)
        outfile.write("ctx.run_template('%s.html')\n" % self.pagename)
        if self in endstates:
            showIndent(outfile, level)
            outfile.write('ctx.set_save_session(0)\n')
            showIndent(outfile, level)
            outfile.write('ctx.remove_session()\n')
        outfile.write("\n\n")
    def generateHtml(self, stateDict):
        filename = '%s.html' % self.pagename
        outfile = file(filename, 'w')
        wrt = outfile.write
        wrt('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">\n')
        wrt('<html>\n')
        wrt('<head>\n')
        wrt('<title>%s</title>\n' % self.pagename)
        wrt('</head>\n')
        wrt('<body>\n')
        wrt('<al-form method="post">\n')
        wrt('<div align="center"><h1>%s</h1></div>\n' % self.pagename)
        previousItems = []
        for transition in self.transitions:
            inputs = transition.getInputs()
            if inputs:
                inputlist = inputs.getInput()
                for input in inputlist:
                    name = input.getName()
                    if not name in previousItems:
                        wrt('  %s: <input type="text" name="%s" value=""><br>\n' % \
                            (name, name))
                        previousItems.append(name)
        wrt('  <hr>\n')
        for transition in self.transitions:
            wrt('  <input type="submit" name="%s" value="%s"><br>\n' % \
                (transition.getButton(), transition.getButton()))
        wrt('</al-form>\n')
        wrt('</body>\n')
        wrt('</html>\n\n')
        outfile.close()
# end class state


class transition:
    def __init__(self, action=None, newstatename='', \
            newstate=None, inputs=None, button=''):
        self.action = action
        self.newstatename = newstatename
        self.newstate = newstate
##         if newstate is None:
##             self.newstate = []
##         else:
##             self.newstate = transitions
        self.inputs = inputs
        self.button = button
    def getAction(self): return self.action
    def setAction(self, action): self.action = action
    def getNewstatename(self): return self.newstatename
    def setNewstatename(self, newstatename): self.newstatename = newstatename
    def getNewstate(self): return self.newstate
    def setNewstate(self, newstate): self.newstate = newstate
    def getButton(self): return self.button
    def setButton(self, button): self.button = button
    def getInputs(self): return self.inputs
    def setInputs(self, inputs): self.inputs = inputs
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<transition>\n')
        level += 1
        if self.inputs:
            self.inputs.export(outfile, level)
        if self.action:
            self.action.export(outfile, level)
        showIndent(outfile, level)
        outfile.write('<newstatename>%s</newstatename>\n' % \
            self.getNewstatename())
        level -= 1
        showIndent(outfile, level)
        outfile.write('</transition>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'action':
                obj = action()
                obj.build(child)
                self.setAction(obj)
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'newstatename':
                newstatename = ''
                for text_ in child.childNodes:
                    newstatename += text_.nodeValue
                self.newstatename = newstatename
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'inputs':
                obj = inputs()
                obj.build(child)
                self.inputs = obj
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'button':
                button = ''
                for text_ in child.childNodes:
                    button += text_.nodeValue
                self.button = button
    def test(self, inputs):
        return 1
    def perform(self):
        self.action.perform(environment)
        return self.newstate
    def linkTransitions(self, stateDict):
        if stateDict.has_key(self.newstatename):
            self.newstate = stateDict[self.newstatename]
    def checkInputs(self, argsDict):
        inputs = self.inputs.getInput()
        for input in inputs:
            if not argsDict.has_key(input.getName()):
                return 0
            if not argsDict[input.getName()] == input.getValue():
                return 0
        return 1
    def reportTransitions(self, outfile, level, stateDict):
        showIndent(outfile, level)
        tostate = None
        if stateDict.has_key(self.newstatename):
            tostate = stateDict[self.newstatename]
        if tostate:
            outfile.write('To state: %s  To page: %s\n' % \
                (tostate.getName(), tostate.getPagename()))
        else:
            outfile.write('*** Error\n')
    def reportPageTransitions(self, outfile, level, stateDict):
        showIndent(outfile, level)
        tostate = None
        if stateDict.has_key(self.newstatename):
            tostate = stateDict[self.newstatename]
        if tostate:
            outfile.write('To page: %s\n' % tostate.getPagename())
        else:
            outfile.write('*** Error\n')
    def generateObjCtrlr(self, outfile, level, stateDict, firstTime):
        if firstTime:
            s1 = "if ctx.req_equals('%s')" % self.button
        else:
            s1 = "elif ctx.req_equals('%s')" % self.button
        if self.inputs:
            inputs = self.inputs.getInput()
        else:
            inputs = ()
        if len(inputs) <= 0:
            s1 += ':\n'
        else:
            s1 += ' and \\\n'
        showIndent(outfile, level)
        outfile.write(s1)
        count = 1
        for input in inputs:
            if count < len(inputs):
                s2 = ' and \\\n'
            else:
                s2 = ':\n'
            s1 = "ctx.get_value('%s') %s '%s'%s" % (input.getName(), \
                input.getTransCondition(), input.getValue(), s2)
            showIndent(outfile, level + 2)
            outfile.write(s1)
            count += 1
        showIndent(outfile, level + 1)
        outfile.write("ctx.set_page('%s')\n"% self.newstate.getPagename())
    def generateModCtrlr(self, outfile, level, stateDict, firstTime):
        if firstTime:
            s1 = "if ctx.req_equals('%s')" % self.button
        else:
            s1 = "elif ctx.req_equals('%s')" % self.button
        if self.inputs:
            inputs = self.inputs.getInput()
        else:
            inputs = ()
        if len(inputs) <= 0:
            s1 += ':\n'
        else:
            s1 += ' and \\\n'
        showIndent(outfile, level)
        outfile.write(s1)
        count = 1
        for input in inputs:
            if count < len(inputs):
                s2 = ' and \\\n'
            else:
                s2 = ':\n'
            s1 = "ctx.get_value('%s') %s '%s'%s" % (input.getName(), \
                input.getTransCondition(), input.getValue(), s2)
            showIndent(outfile, level + 2)
            outfile.write(s1)
            count += 1
        showIndent(outfile, level + 1)
        outfile.write("ctx.set_page('%s')\n"% self.newstate.getPagename())
##     def generateModCtrlr(self, outfile, level, stateDict, firstTime):
##         showIndent(outfile, level)
##         if firstTime:
##             outfile.write("if ctx.req_equals('%s'):\n" % \
##                 self.newstate.getPagename())
##         else:
##             outfile.write("elif ctx.req_equals('%s'):\n" % \
##                 self.newstate.getPagename())
##         showIndent(outfile, level)
##         outfile.write("    ctx.set_page('%s')\n"% self.newstate.getPagename())
# end class transition


class action:
    def __init__(self, source='', compiled=''):
        self.source = source
        self.compiled = compiled
    def getSource(self): return self.source
    def setSource(self, source): self.source = source
    def getCompiled(self): return self.compiled
    def setCompiled(self, compiled): self.compiled = compiled
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<action>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<source>%s</source>\n' % self.getSource())
        showIndent(outfile, level)
        outfile.write('<compiled>%s</compiled>\n' % self.getCompiled())
        level -= 1
        showIndent(outfile, level)
        outfile.write('</action>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'source':
                source = ''
                for text_ in child.childNodes:
                    source += text_.nodeValue
                self.source = source
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'compiled':
                compiled = ''
                for text_ in child.childNodes:
                    compiled += text_.nodeValue
                self.compiled = compiled
    def perform(self, environment):
        exec self.compiled in environment
# end class action


class endstatenames:
    def __init__(self, endstatenames=None):
        if endstatenames is None:
            self.endstatenames = []
        else:
            self.endstatenames = endstatenames
    def getEndstatenames(self): return self.endstatenames
    def addEndstatenames(self, endstatenames):
        self.endstatename.append(endstatename)
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<endstatenames>\n')
        level += 1
        for endstatename in self.endstatenames:
            showIndent(outfile, level)
            outfile.write('<endstatename>%s</endstatename>\n' % endstatename)
        level -= 1
        showIndent(outfile, level)
        outfile.write('</endstatenames>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'endstatename':
                endstatename = ''
                for text_ in child.childNodes:
                    endstatename += text_.nodeValue
                self.endstatenames.append(endstatename)
# end class endstatenames


class inputs:
    def __init__(self, input=None):
        if input is None:
            self.input = []
        else:
            self.input = input
    def getInput(self): return self.input
    def addInput(self, input): self.input.append(input)
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<inputs>\n')
        level += 1
        for input in self.input:
            input.export(outfile, level)
        level -= 1
        showIndent(outfile, level)
        outfile.write('</inputs>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'input':
                obj = input()
                obj.build(child)
                self.input.append(obj)
# end class inputs


class input:
    transTable = { \
        'eq': '==', \
        'lt': '<', \
        'gt': '>', \
        'le': '<=', \
        'ge': '>=', \
        'ne': '!=' \
        }
    def __init__(self, name='', value='', condition='equal'):
        self.name = name
        self.value = value
        self.condition = condition
    def getName(self): return self.name
    def setName(self, name): self.name = name
    def getValue(self): return self.value
    def setValue(self, value): self.value = value
    def getCondition(self): return self.condition
    def setCondition(self, condition): self.condition = condition
    def getTransCondition(self):
        if self.condition in self.transTable:
            return self.transTable[self.condition]
        else:
            return '**Missing condition**'
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<input>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<name>%s</name>\n' % self.getName())
        showIndent(outfile, level)
        outfile.write('<value>%s</value>\n' % self.getValue())
        level -= 1
        showIndent(outfile, level)
        outfile.write('</input>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'name':
                name = ''
                for text_ in child.childNodes:
                    name += text_.nodeValue
                self.name = name
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'value':
                value = ''
                for text_ in child.childNodes:
                    value += text_.nodeValue
                self.value = value
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'condition':
                condition = ''
                for text_ in child.childNodes:
                    condition += text_.nodeValue
                self.condition = condition
# end class input

#
# Exporting and reporting.
#

def export(inFileName, outFile):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    rootObj.export(outFile, 0)


def reportTransitions(inFileName, outFile):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    rootObj.reportTransitions(outFile, 0)


def reportPageTransitions(inFileName, outFile):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    rootObj.reportPageTransitions(outFile, 0)


def reportStates(inFileName, outFile):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    rootObj.reportStates(outFile, 0)

# End -- Exporting and reporting.


#
# Generate Albatross modular application.
#

def generateModularController(inFileName, outfile, pages):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    level = 0
    generateModCtrlrHeader(outfile)
    #rootObj.generateModularController(outfile, level)
    generateModCtrlrFooter(outfile, rootObj)
    outfile.close()
    rootObj.generateModCtrlr(outfile, level, pages)


def generateModCtrlrHeader(outfile):
    outfile.write('#!/usr/bin/env python\n\n')
    outfile.write(\
        'from albatross import ModularSessionApp, SessionAppContext\n')
    outfile.write(\
        '#from albatross import ModularSessionFileApp, SessionFileAppContext\n')
    outfile.write('from albatross.cgiapp import Request\n')
    outfile.write('\n\n')


def generateModCtrlrFooter(outfile, rootObj):
    wrt = outfile.write
    wrt("class AppContext(SessionAppContext):\n")
    wrt("    def __init__(self, app):\n")
    wrt("        SessionAppContext.__init__(self, app)\n")
    wrt("\n")
    wrt("\n")
    wrt("class App(ModularSessionApp):\n")
    wrt("    def __init__(self):\n")
    wrt("        ModularSessionApp.__init__(self,\n")
    wrt("            base_url = 'dispatch.py',\n")
    wrt("            module_path = '.',\n")
    wrt("            template_path = '.',\n")
    name = rootObj.getStartstate().getPagename()
    wrt("            start_page = '%s',\n" % name)
    wrt("            secret = 'finch',\n")
    wrt("            session_appid = 'app3')\n")
    wrt("            #session_dir= '/w2/Python/albatross-0.06/session-server/SessionData')\n")
    wrt("    def create_context(self):\n")
    wrt("        return AppContext(self)\n")
    wrt("\n")
    wrt("\n")
    wrt("if __name__ == '__main__':\n")
    wrt("    app = App()\n")
    wrt("    app.run(Request())\n")
    wrt("\n")
    wrt("\n")

# End -- Generate Albatross modular application.


#
# Generate Albatross object-based application.
#

def generateObjectController(inFileName, outfile, pages):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    level = 0
    generateObjCtrlrHeader(outfile)
    rootObj.generateObjCtrlr(outfile, level, pages)
    generateObjCtrlrFooter(outfile, rootObj)


def generateObjCtrlrHeader(outfile):
    outfile.write('#!/usr/bin/env python\n\n')
    outfile.write(\
        'from albatross import SimpleSessionApp, SessionAppContext\n')
    outfile.write(\
        '#from albatross import SimpleSessionFileApp, SessionFileAppContext\n')
    outfile.write('from albatross.cgiapp import Request\n')
    outfile.write('\n\n')


def generateObjCtrlrFooter(outfile, rootObj):
    wrt = outfile.write
    wrt("class AppContext(SessionAppContext):\n")
    wrt("    def __init__(self, app):\n")
    wrt("        SessionAppContext.__init__(self, app)\n")
    wrt("\n")
    wrt("\n")
    wrt("class App(SimpleSessionApp):\n")
    wrt("    def __init__(self):\n")
    wrt("        SimpleSessionApp.__init__(self,\n")
    wrt("            base_url = 'dispatch.py',\n")
    wrt("            template_path = '.',\n")
    name = rootObj.getStartstate().getPagename()
    wrt("            start_page = '%s',\n" % name)
    wrt("            secret = 'finch',\n")
    wrt("            session_appid = 'app3')\n")
    wrt("            #session_dir= '/w2/Python/albatross-0.06/session-server/SessionData')\n")
    page_names = ''
    for state in rootObj.getStates():
        page_names += state.getPagename() + ', '
    wrt("        for page_class in (%s):\n" % page_names)
    wrt("            self.register_page(page_class.name, page_class())\n")
    wrt("    def create_context(self):\n")
    wrt("        return AppContext(self)\n")
    wrt("\n")
    wrt("\n")
    wrt("if __name__ == '__main__':\n")
    wrt("    app = App()\n")
    wrt("    app.run(Request())\n")
    wrt("\n")
    wrt("\n")

# End -- Generate Albatross object-based application.


def loadFSM(inFileName):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    return rootObj


USAGE_TEXT = """
Usage: python <Parser>.py <in_xml_file>
"""

def usage():
    print USAGE_TEXT
    sys.exit(-1)


def parse(inFileName):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = fsm()
    rootObj.build(rootNode)
    rootObj.linkStates()
    rootObj.export(sys.stdout, 0)
    rootObj.show()
    return rootObj


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        usage()
    parse(args[0])


if __name__ == '__main__':
    main()
    #import pdb
    #pdb.run('main()')


