#!/usr/bin/env python

import sys, getopt, os.path
import fsmlib

def generateObj(inFileName, outFile, pages):
    fsmlib.generateObjectController(inFileName, outFile, pages)


def generateMod(inFileName, outFile, pages):
    fsmlib.generateModularController(inFileName, outFile, pages)


USAGE_TEXT = """
Usage:
    python testGen.py <command> [ <option> ] <in_file_name> <out_file_name>

Commands:
    -o    Generate object-based Albatross application.
    -m    Generate modular Albatross application.
    -h    Display this help.

Options:
    -p    Generate HTML template files.

Examples:
    python testGen.py -o fsm_app1.xml dispatch.py
        Generate object-based Albatross application.

    python testGen.py -m -p fsm_app2.xml dispatch.py
        Generate modular Albatross application and generate
            HTML template files.

"""

def usage():
    print USAGE_TEXT
    sys.exit(-1)


def makeFile(outFileName):
    outFile = None
    if os.path.exists(outFileName):
        reply = raw_input('File %s exists.  Overwrite? (y/n): ' % outFileName)
        if reply == 'y':
            outFile = file(outFileName, 'w')
    else:
        outFile = file(outFileName, 'w')
    return outFile

def main():
    args = sys.argv[1:]
    opts, args = getopt.getopt(args, 'moph')
    pages = 0
    option = ''
    for opt, arg in opts:
        if opt == '-o':
            option = '-o'
        elif opt == '-m':
            option = '-m'
        elif opt == '-p':
            pages = 1
        elif opt == '-h':
            option = '-h'
    if option == '-o':
        if len(args) == 2:
            inFileName = args[0]
            outFileName = args[1]
            outFile = makeFile(outFileName)
            if not outFile:
                return
            generateObj(inFileName, outFile, pages)
            outFile.close()
        else:
            usage()
    elif option == '-m':
        if len(args) == 2:
            inFileName = args[0]
            outFileName = args[1]
            outFile = makeFile(outFileName)
            if not outFile:
                return
            generateMod(inFileName, outFile, pages)
        else:
            usage()
    elif option == '-h':
        usage()
    else:
        usage()


if __name__ == '__main__':
    main()
    #import pdb
    #pdb.run('main()')


